@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +37,7 @@ THE SOFTWARE.
 #include "OgreBone.h"
 #include "OgreIteratorWrappers.h"
 #include "OgreMesh.h"
+#include "OgreHeaderPrefix.h"
 
 namespace Ogre {
 
@@ -72,7 +73,7 @@ namespace Ogre {
 			Therefore you won't gain as much benefit from the batching if you 
 			use many different materials; try to keep the number down.
 	@par
-		The bounding box information is computed whith object position only. 
+		The bounding box information is computed with object position only. 
 		It doesn't take account of the object orientation. 
 	@par
 		The LOD settings of both the Mesh and the Materials used in 
@@ -90,7 +91,7 @@ namespace Ogre {
 		There are 2 ways you can add geometry to this class; you can add
 		Entity objects directly with predetermined positions, scales and 
 		orientations, or you can add an entire SceneNode and it's subtree, 
-		including all the objects attached to it. Once you've added everthing
+		including all the objects attached to it. Once you've added everything
 		you need to, you have to call build() the fix the geometry in place. 
 	@par
 		You should not construct instances of this class directly; instead, call 
@@ -104,7 +105,7 @@ namespace Ogre {
 	class _OgreExport  InstancedGeometry : public BatchedGeometryAlloc
 	{
 	public:
-		/** Struct holding geometry optimised per SubMesh / lod level, ready
+		/** Struct holding geometry optimised per SubMesh / LOD level, ready
 			for copying to instances. 
 		@remarks
 			Since we're going to be duplicating geometry lots of times, it's
@@ -216,10 +217,17 @@ namespace Ogre {
 					}
 				}
 			}
+
+			void _initGeometryBucket(const VertexData* vData, const IndexData* iData);
+			void _initGeometryBucket(GeometryBucket* bucket);
+
 		public:
 			GeometryBucket(MaterialBucket* parent, const String& formatString, 
 				const VertexData* vData, const IndexData* iData);
+			GeometryBucket(const String& name, MaterialBucket* parent, const String& formatString, 
+				const VertexData* vData, const IndexData* iData);
 			GeometryBucket(MaterialBucket* parent,const String& formatString,GeometryBucket*bucket);
+			GeometryBucket(const String& name, MaterialBucket* parent,const String& formatString,GeometryBucket*bucket);
 			virtual ~GeometryBucket();
 			MaterialBucket* getParent(void) { return mParent; }
 			Real getBoundingRadius(void) const;
@@ -237,14 +245,14 @@ namespace Ogre {
 			bool getCastsShadows(void) const;
 			String getFormatString(void) const;
 			/** Try to assign geometry to this bucket.
-			@returns false if there is no room left in this bucket
+			@return false if there is no room left in this bucket
 			*/
 			bool assign(QueuedGeometry* qsm);
 			/// Build
 			void build();
 			/// Dump contents for diagnostics
 			void dump(std::ofstream& of) const;
-			/// retun the BoundingBox information. Usefull when cloning the batch instance.
+			/// Return the BoundingBox information. Useful when cloning the batch instance.
 			AxisAlignedBox & getAABB(void){return mAABB;}
 			/// @copydoc MovableObject::visitRenderables
 			void visitRenderables(Renderable::Visitor* visitor, bool debugRenderables);
@@ -383,7 +391,7 @@ namespace Ogre {
 			BatchInstance* mParent;
 			/// LOD level (0 == full LOD)
 			unsigned short mLod;
-			/// lod value at which this LOD starts to apply (squared)
+			/// LOD value at which this LOD starts to apply (squared)
 			Real mLodValue;
 			/// Lookup of Material Buckets in this BatchInstance
 			MaterialBucketMap mMaterialBucketMap;
@@ -393,9 +401,9 @@ namespace Ogre {
 			LODBucket(BatchInstance* parent, unsigned short lod, Real lodValue);
 			virtual ~LODBucket();
 			BatchInstance* getParent(void) { return mParent; }
-			/// Get the lod index
+			/// Get the LOD index
 			ushort getLod(void) const { return mLod; }
-			/// Get the lod value
+			/// Get the LOD value
 			Real getLodValue(void) const { return mLodValue; }
 			/// Assign a queued submesh to this bucket, using specified mesh LOD
 			void assign(QueuedSubMesh* qsm, ushort atLod);
@@ -448,24 +456,24 @@ namespace Ogre {
 
 			ObjectsMap mInstancesMap;
 		public:
-			/// Lod values as built up - use the max at each level
+			/// LOD values as built up - use the max at each level
 			Mesh::LodValueList mLodValues;
 			/// Local AABB relative to BatchInstance centre
 			AxisAlignedBox mAABB;
 			/// Local bounding radius
 			Real mBoundingRadius;
-			/// The current lod level, as determined from the last camera
+			/// The current LOD level, as determined from the last camera
 			ushort mCurrentLod;
-			/// Current lod value, passed on to do material lod later
+			/// Current LOD value, passed on to do material LOD later
 			Real mLodValue;
-            /// Current camera, passed on to do material lod later
+            /// Current camera, passed on to do material LOD later
             Camera *mCamera;
             /// Cached squared view depth value to avoid recalculation by GeometryBucket
             Real mSquaredViewDepth;
 		protected:
 			/// List of LOD buckets			
 			LODBucketList mLodBucketList;
-            /// Lod strategy reference
+            /// LOD strategy reference
             const LodStrategy *mLodStrategy;
 
 		public:
@@ -522,7 +530,7 @@ namespace Ogre {
 			each axis.
 		*/
 		typedef map<uint32, BatchInstance*>::type BatchInstanceMap;
-		/** Simple vectors where are stored all the renderoperations of the Batch.
+		/** Simple vectors where are stored all the render operations of the Batch.
 			This vector is used when we want to delete the batch, in order to delete only one time each
 			render operation.
 
@@ -666,7 +674,7 @@ namespace Ogre {
 			this InstancedGeometry if you like. The Entity passed in is simply 
 			used as a definition.
 		@note Must be called before 'build'.
-        @note All added entities must use the same lod strategy.
+        @note All added entities must use the same LOD strategy.
 		@param ent The Entity to use as a definition (the Mesh and Materials 
 			referenced will be recorded for the build call).
 		@param position The world position at which to add this Entity
@@ -692,7 +700,7 @@ namespace Ogre {
 			versions! We don't do this for you incase you are preparing this 
 			in advance and so don't want the originals detached yet. 
 		@note Must be called before 'build'.
-        @note All added entities must use the same lod strategy.
+        @note All added entities must use the same LOD strategy.
 		@param node Pointer to the node to use to provide a set of Entity 
 			templates
 		*/
@@ -712,7 +720,7 @@ namespace Ogre {
 			/** Add a new batch instance
 		@remarks
 				This method add a new instance of the whole batch, by creating a new 
-				BatchInstance, containing new lod buckets, material buckets and geometry buckets.
+				BatchInstance, containing new LOD buckets, material buckets and geometry buckets.
 				The new geometry buckets will use the same buffers as the base bucket.
 		@note
 			no note
@@ -803,7 +811,7 @@ namespace Ogre {
 			the sparseness of population is no issue when it comes to rendering.
 			The default is Vector3(0,0,0).
 		@note Must be called before 'build'.
-		@param size Vector3 expressing the 3D origin of the geometry.
+		@param origin Vector3 expressing the 3D origin of the geometry.
 		*/
 		virtual void setOrigin(const Vector3& origin) { mOrigin = origin; }
 		/** Gets the origin of this geometry. */
@@ -879,5 +887,7 @@ namespace Ogre {
 	/** @} */
 }
 
+#include "OgreHeaderSuffix.h"
+
 #endif
 
