@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -36,6 +36,7 @@ THE SOFTWARE.
 #include "OgreTextureUnitState.h"
 #include "OgreCommon.h"
 
+#include "OgreMaterialManager.h"
 #include "OgreRenderOperation.h"
 #include "OgreRenderSystemCapabilities.h"
 #include "OgreRenderTarget.h"
@@ -45,6 +46,7 @@ THE SOFTWARE.
 #include "OgreGpuProgram.h"
 #include "OgrePlane.h"
 #include "OgreIteratorWrappers.h"
+#include "OgreHeaderPrefix.h"
 
 namespace Ogre
 {
@@ -55,6 +57,8 @@ namespace Ogre
 	*  @{
 	*/
 
+	typedef vector<DepthBuffer*>::type DepthBufferVec;
+	typedef map< uint16, DepthBufferVec >::type DepthBufferMap;
 	typedef map< String, RenderTarget * >::type RenderTargetMap;
 	typedef multimap<uchar, RenderTarget * >::type RenderTargetPriorityMap;
 
@@ -73,7 +77,7 @@ namespace Ogre
 		/// Projective texture
 		TEXCALC_PROJECTIVE_TEXTURE
 	};
-	/// Enum describing the various actions which can be taken onthe stencil buffer
+	/// Enum describing the various actions which can be taken on the stencil buffer
 	enum StencilOperation
 	{
 		/// Leave the stencil buffer unchanged
@@ -150,7 +154,7 @@ namespace Ogre
 		This routine is called automatically by the default
 		configuration dialogue produced by Root::showConfigDialog
 		or may be used by the caller for custom settings dialogs
-		@returns
+		@return
 		A 'map' of options, i.e. a list of options which is also
 		indexed by option name.
 		*/
@@ -200,18 +204,31 @@ namespace Ogre
 		automatically, based on settings chosen so far. This saves
 		an extra call to _createRenderWindow
 		for the main render window.
-		@par
-		If an application has more specific window requirements,
-		however (e.g. a level design app), it should specify false
-		for this parameter and do it manually.
-		@returns
+		@param
+		windowTitle Sets the app window title
+		@return
 		A pointer to the automatically created window, if requested, otherwise null.
 		*/
 		virtual RenderWindow* _initialise(bool autoCreateWindow, const String& windowTitle = "OGRE Render Window");
 
+		/*
+		Returns whether under the current render system buffers marked as TU_STATIC can be locked for update
+		@remarks
+		Needed in the implementation of DirectX9 with DirectX9Ex driver
+		*/
+		virtual bool isStaticBufferLockable() const { return true; }
 
 		/** Query the real capabilities of the GPU and driver in the RenderSystem*/
 		virtual RenderSystemCapabilities* createRenderSystemCapabilities() const = 0;
+ 
+		/** Get a pointer to the current capabilities being used by the RenderSystem.
+		@remarks
+		The capabilities may be modified using this pointer, this will only have an effect
+		before the RenderSystem has been initialised. It's intended use is to allow a
+		listener of the RenderSystemCapabilitiesCreated event to customise the capabilities
+		on the fly before the RenderSystem is initialised.
+		*/
+		RenderSystemCapabilities* getMutableCapabilities(){ return mCurrentCapabilities; }
 
 		/** Force the render system to use the special capabilities. Can only be called
 		*    before the render system has been fully initializer (before createWindow is called) 
@@ -253,7 +270,7 @@ namespace Ogre
 		*/
 		void setWBufferEnabled(bool enabled);
 
-		/** Returns true if the renderer will try to use W-buffers when avalible.
+		/** Returns true if the renderer will try to use W-buffers when available.
 		*/
 		bool getWBufferEnabled(void) const;
 
@@ -330,7 +347,10 @@ namespace Ogre
 		<tr>
 			<td>externalWindowHandle</td>
 			<td>Win32: HWND as integer<br/>
-			    GLX: poslong:posint:poslong (display*:screen:windowHandle) or poslong:posint:poslong:poslong (display*:screen:windowHandle:XVisualInfo*)</td>
+			    GLX: poslong:posint:poslong (display*:screen:windowHandle) or poslong:posint:poslong:poslong (display*:screen:windowHandle:XVisualInfo*)<br/>
+                OS X: WindowRef for Carbon or NSWindow for Cocoa address as an integer
+                iOS: UIWindow address as an integer
+            </td>
 			<td>0 (none)</td>
 			<td>External window handle, for embedding the OGRE render in an existing window</td>
 			<td>&nbsp;</td>
@@ -367,6 +387,7 @@ namespace Ogre
 			<td>String: "cocoa" or "carbon"</td>
 			<td>"carbon"</td>
 			<td>Specifies the type of rendering window on the Mac Platform.</td>
+            <td>Mac OS X Specific</td>
 			<td>&nbsp;</td>
 		 </tr>
 		 <tr>
@@ -381,6 +402,7 @@ namespace Ogre
 				In short, by setting this flag to "true" the Ogre::Root::createRenderWindow interprets the "externalWindowHandle" as a NSView*
 				instead of an OgreView*. See OgreOSXCocoaView.h/mm.
 			</td>
+            <td>Mac OS X Specific</td>
 			<td>&nbsp;</td>
 		 </tr>
          <tr>
@@ -392,8 +414,41 @@ namespace Ogre
                  native resolution is 960 x 640.  Windows are always 320 x 480, if you would like to limit the display
                  to 720 x 480, specify 1.5 as the scaling factor.
              </td>
+             <td>iOS Specific</td>
              <td>&nbsp;</td>
 		 </tr>
+         <tr>
+             <td>externalViewHandle</td>
+             <td>UIView pointer as an integer</td>
+             <td>0</td>
+             <td>External view handle, for rendering OGRE render in an existing view</td>
+             <td>iOS Specific</td>
+             <td>&nbsp;</td>
+         </tr>
+         <tr>
+             <td>externalViewControllerHandle</td>
+             <td>UIViewController pointer as an integer</td>
+             <td>0</td>
+             <td>External view controller handle, for embedding OGRE in an existing view controller</td>
+             <td>iOS Specific</td>
+             <td>&nbsp;</td>
+         </tr>
+         <tr>
+             <td>externalSharegroup</td>
+             <td>EAGLSharegroup pointer as an integer</td>
+             <td>0</td>
+             <td>External sharegroup, used to shared GL resources between contexts</td>
+             <td>iOS Specific</td>
+             <td>&nbsp;</td>
+         </tr>
+         <tr>
+             <td>Full Screen</td>
+             <td>true, false</td>
+             <td>false</td>
+             <td>Specify whether to create the window in full screen mode</td>
+             <td>OS X Specific</td>
+             <td>&nbsp;</td>
+         </tr>
          <tr>
 			<td>FSAA</td>
 			<td>Positive integer (usually 0, 2, 4, 8, 16)</td>
@@ -462,6 +517,14 @@ namespace Ogre
 			colour space on rendering to the window.</td>
 			<td>&nbsp;</td>
 		</tr>
+		<tr>
+			<td>enableDoubleClick</td>
+			<td>true, false</td>
+			<td>false</td>
+			<td>Enable the window to keep track and transmit double click messages.</td>
+			<td>Win32 Specific</td>
+		</tr>
+		
 		*/
 		virtual RenderWindow* _createRenderWindow(const String &name, unsigned int width, unsigned int height, 
 			bool fullScreen, const NameValuePairList *miscParams = 0) = 0;
@@ -478,7 +541,7 @@ namespace Ogre
 		See _createRenderWindow for details about each member.		
 		@param
 		createdWindows This array will hold the created render windows.
-		@returns
+		@return
 		true on success.		
 		*/
 		virtual bool _createRenderWindows(const RenderWindowDescriptionList& renderWindowDescriptions, 
@@ -523,24 +586,39 @@ namespace Ogre
 		*/
 		virtual String getErrorDescription(long errorNumber) const = 0;
 
-		/** Defines whether or now fullscreen render windows wait for the vertical blank before flipping buffers.
-		@remarks
-		By default, all rendering windows wait for a vertical blank (when the CRT beam turns off briefly to move
-		from the bottom right of the screen back to the top left) before flipping the screen buffers. This ensures
-		that the image you see on the screen is steady. However it restricts the frame rate to the refresh rate of
-		the monitor, and can slow the frame rate down. You can speed this up by not waiting for the blank, but
-		this has the downside of introducing 'tearing' artefacts where part of the previous frame is still displayed
-		as the buffers are switched. Speed vs quality, you choose.
-		@note
-		Has NO effect on windowed mode render targets. Only affects fullscreen mode.
-		@param
-		enabled If true, the system waits for vertical blanks - quality over speed. If false it doesn't - speed over quality.
+		/** Returns the global instance vertex buffer.
 		*/
-		void setWaitForVerticalBlank(bool enabled);
+        HardwareVertexBufferSharedPtr getGlobalInstanceVertexBuffer() const;
+		/** Sets the global instance vertex buffer.
+		*/
+        void setGlobalInstanceVertexBuffer(const HardwareVertexBufferSharedPtr &val);
+		/** Gets vertex declaration for the global vertex buffer for the global instancing
+		*/
+        VertexDeclaration* getGlobalInstanceVertexBufferVertexDeclaration() const;
+		/** Sets vertex declaration for the global vertex buffer for the global instancing
+		*/
+        void setGlobalInstanceVertexBufferVertexDeclaration( VertexDeclaration* val);
+		/** Gets the global number of instances.
+		*/
+        size_t getGlobalNumberOfInstances() const;
+		/** Sets the global number of instances.
+		*/
+        void setGlobalNumberOfInstances(const size_t val);
 
-		/** Returns true if the system is synchronising frames with the monitor vertical blank.
+		/** Sets if fixed pipeline rendering is enabled on the system.
 		*/
-		bool getWaitForVerticalBlank(void) const;
+		void setFixedPipelineEnabled(bool enabled);
+
+		/** Returns true if fixed pipeline rendering is enabled on the system.
+		*/
+		bool getFixedPipelineEnabled(void) const;
+
+		/** Retrieves an existing DepthBuffer or creates a new one suited for the given RenderTarget
+			and sets it.
+			@remarks
+				RenderTarget's pool ID is respected. @see RenderTarget::setDepthBufferPool()
+		*/
+		virtual void setDepthBufferFor( RenderTarget *renderTarget );
 
 		// ------------------------------------------------------------------------
 		//                     Internal Rendering Access
@@ -718,6 +796,20 @@ namespace Ogre
 		*/
 		virtual void _setTextureUnitFiltering(size_t unit, FilterType ftype, FilterOptions filter) = 0;
 
+		/** Sets whether the compare func is enabled or not for this texture unit 
+		@param unit The texture unit to set the filtering options for
+		@param compare The state (enabled/disabled)
+		*/
+		virtual void _setTextureUnitCompareEnabled(size_t unit, bool compare) = 0;
+
+
+		/** Sets the compare function to use for a given texture unit
+		@param unit The texture unit to set the filtering options for
+		@param function The comparison function
+		*/
+		virtual void _setTextureUnitCompareFunction(size_t unit, CompareFunction function) = 0;
+
+
 		/** Sets the maximal anisotropy for the specified texture unit.*/
 		virtual void _setTextureLayerAnisotropy(size_t unit, unsigned int maxAnisotropy) = 0;
 
@@ -744,7 +836,7 @@ namespace Ogre
 		virtual void _setTextureMatrix(size_t unit, const Matrix4& xform) = 0;
 
 		/** Sets the global blending factors for combining subsequent renders with the existing frame contents.
-		The result of the blending operation is:</p>
+		The result of the blending operation is:
 		<p align="center">final = (texture * sourceFactor) + (pixel * destFactor)</p>
 		Each of the factors is specified as one of a number of options, as specified in the SceneBlendFactor
 		enumerated type.
@@ -756,7 +848,7 @@ namespace Ogre
 		virtual void _setSceneBlending(SceneBlendFactor sourceFactor, SceneBlendFactor destFactor, SceneBlendOperation op = SBO_ADD) = 0;
 
 		/** Sets the global blending factors for combining subsequent renders with the existing frame contents.
-		The result of the blending operation is:</p>
+		The result of the blending operation is:
 		<p align="center">final = (texture * sourceFactor) + (pixel * destFactor)</p>
 		Each of the factors is specified as one of a number of options, as specified in the SceneBlendFactor
 		enumerated type.
@@ -773,7 +865,7 @@ namespace Ogre
 		/** Sets the global alpha rejection approach for future renders.
 		By default images are rendered regardless of texture alpha. This method lets you change that.
 		@param func The comparison function which must pass for a pixel to be written.
-		@param val The value to compare each pixels alpha value to (0-255)
+		@param value The value to compare each pixels alpha value to (0-255)
 		@param alphaToCoverage Whether to enable alpha to coverage, if supported
 		*/
 		virtual void _setAlphaRejectSettings(CompareFunction func, unsigned char value, bool alphaToCoverage) = 0;
@@ -782,6 +874,25 @@ namespace Ogre
 			relative to a different origin.
 		*/
 		virtual void _setTextureProjectionRelativeTo(bool enabled, const Vector3& pos);
+
+		/** Creates a DepthBuffer that can be attached to the specified RenderTarget
+			@remarks
+				It doesn't attach anything, it just returns a pointer to a new DepthBuffer
+				Caller is responsible for putting this buffer into the right pool, for
+				attaching, and deleting it. Here's where API-specific magic happens.
+				Don't call this directly unless you know what you're doing.
+		*/
+		virtual DepthBuffer* _createDepthBufferFor( RenderTarget *renderTarget ) = 0;
+
+		/** Removes all depth buffers. Should be called on device lost and shutdown
+			@remarks
+				Advanced users can call this directly with bCleanManualBuffers=false to
+				remove all depth buffers created for RTTs; when they think the pool has
+				grown too big or they've used lots of depth buffers they don't need anymore,
+				freeing GPU RAM.
+		*/
+		void _cleanupDepthBuffers( bool bCleanManualBuffers=true );
+
 		/**
 		* Signifies the beginning of a frame, i.e. the start of rendering on a single viewport. Will occur
 		* several times per complete frame if multiple viewports exist.
@@ -814,7 +925,7 @@ namespace Ogre
 		rendering operations. This viewport is aware of it's own
 		camera and render target. Must be implemented by subclass.
 
-		@param target Pointer to the appropriate viewport.
+		@param vp Pointer to the appropriate viewport.
 		*/
 		virtual void _setViewport(Viewport *vp) = 0;
 		/** Get the current active viewport for rendering. */
@@ -1042,8 +1153,11 @@ namespace Ogre
 		start up too.
 		@param func The comparison function applied.
 		@param refValue The reference value used in the comparison
-		@param mask The bitmask applied to both the stencil value and the reference value 
+		@param compareMask The bitmask applied to both the stencil value and the reference value 
 		before comparison
+		@param writeMask The bitmask the controls which bits from refValue will be written to 
+		stencil buffer (valid for operations such as SOP_REPLACE).
+		the stencil
 		@param stencilFailOp The action to perform when the stencil check fails
 		@param depthFailOp The action to perform when the stencil check passes, but the
 		depth buffer check still fails
@@ -1053,7 +1167,7 @@ namespace Ogre
 		and the inverse of them will happen for back faces (keep remains the same).
 		*/
 		virtual void setStencilBufferParams(CompareFunction func = CMPF_ALWAYS_PASS, 
-			uint32 refValue = 0, uint32 mask = 0xFFFFFFFF, 
+			uint32 refValue = 0, uint32 compareMask = 0xFFFFFFFF, uint32 writeMask = 0xFFFFFFFF, 
 			StencilOperation stencilFailOp = SOP_KEEP, 
 			StencilOperation depthFailOp = SOP_KEEP,
 			StencilOperation passOp = SOP_KEEP, 
@@ -1100,6 +1214,17 @@ namespace Ogre
 		*/
 		virtual const DriverVersion& getDriverVersion(void) const { return mDriverVersion; }
 
+        /** Returns the default material scheme used by the render system.
+            Systems that use the RTSS to emulate a fixed function pipeline 
+            (e.g. OpenGL ES 2, GL3+, DX11) need to override this function to return
+            the default material scheme of the RTSS ShaderGenerator.
+         
+            This is currently only used to set the default material scheme for
+            viewports.  It is a necessary step on these render systems for
+            render textures to be rendered into properly.
+		*/
+		virtual const String& _getDefaultViewportMaterialScheme(void) const;
+
 		/** Binds a given GpuProgram (but not the parameters). 
 		@remarks Only one GpuProgram of each type can be bound at once, binding another
 		one will simply replace the existing one.
@@ -1126,6 +1251,13 @@ namespace Ogre
 		/** Returns whether or not a Gpu program of the given type is currently bound. */
 		virtual bool isGpuProgramBound(GpuProgramType gptype);
 
+        /**
+         * Gets the native shading language version for this render system.
+         * Formatted so that it can be used within a shading program. 
+         * For example, OpenGL 3.2 would return 150, while 3.3 would return 330
+         */
+        uint16 getNativeShadingLanguageVersion() const { return mNativeShadingLanguageVersion; }
+
 		/** Sets the user clipping region.
 		*/
 		virtual void setClipPlanes(const PlaneList& clipPlanes);
@@ -1151,7 +1283,7 @@ namespace Ogre
 		virtual void _updateAllRenderTargets(bool swapBuffers = true);
 		/** Internal method for swapping all the buffers on all render targets,
 		if _updateAllRenderTargets was called with a 'false' parameter. */
-		virtual void _swapAllRenderTargetBuffers(bool waitForVsync = true);
+		virtual void _swapAllRenderTargetBuffers();
 
 		/** Sets whether or not vertex windings set should be inverted; this can be important
 		for rendering reflections. */
@@ -1162,7 +1294,7 @@ namespace Ogre
 		*/
 		virtual bool getInvertVertexWinding(void) const;
 
-		/** Sets the 'scissor region' ie the region of the target in which rendering can take place.
+		/** Sets the 'scissor region' i.e. the region of the target in which rendering can take place.
 		@remarks
 		This method allows you to 'mask off' rendering in all but a given rectangular area
 		as identified by the parameters to this method.
@@ -1345,8 +1477,38 @@ namespace Ogre
 		@see Root::getDisplayMonitorCount
 		*/
 		virtual unsigned int getDisplayMonitorCount() const = 0;
+
+        /**
+        * This marks the beginning of an event for GPU profiling.
+        */
+        virtual void beginProfileEvent( const String &eventName ) = 0;
+
+        /**
+        * Ends the currently active GPU profiling event.
+        */
+        virtual void endProfileEvent( void ) = 0;
+
+        /**
+        * Marks an instantaneous event for graphics profilers.  
+        * This is equivalent to calling @see beginProfileEvent and @see endProfileEvent back to back.
+        */
+        virtual void markProfileEvent( const String &event ) = 0;
+
+		/** Determines if the system has anisotropic mip map filter support
+		*/
+		virtual bool hasAnisotropicMipMapFilter() const = 0;
+
+		/** Gets a custom (maybe platform-specific) attribute.
+        @remarks This is a nasty way of satisfying any API's need to see platform-specific details.
+        @param name The name of the attribute.
+        @param pData Pointer to memory of the right kind of structure to receive the info.
+        */
+		virtual void getCustomAttribute(const String& name, void* pData);
+
 	protected:
 
+		/** DepthBuffers to be attached to render targets */
+		DepthBufferMap	mDepthBufferPool;
 
 		/** The render targets. */
 		RenderTargetMap mRenderTargets;
@@ -1354,10 +1516,14 @@ namespace Ogre
 		RenderTargetPriorityMap mPrioritisedRenderTargets;
 		/** The Active render target. */
 		RenderTarget * mActiveRenderTarget;
+
 		/** The Active GPU programs and gpu program parameters*/
 		GpuProgramParametersSharedPtr mActiveVertexGpuProgramParameters;
 		GpuProgramParametersSharedPtr mActiveGeometryGpuProgramParameters;
 		GpuProgramParametersSharedPtr mActiveFragmentGpuProgramParameters;
+		GpuProgramParametersSharedPtr mActiveTesselationHullGpuProgramParameters;
+		GpuProgramParametersSharedPtr mActiveTesselationDomainGpuProgramParameters;
+		GpuProgramParametersSharedPtr mActiveComputeGpuProgramParameters;
 
 		// Texture manager
 		// A concrete class of this will be created and
@@ -1370,8 +1536,6 @@ namespace Ogre
 
 		CullingMode mCullingMode;
 
-		bool mVSync;
-		unsigned int mVSyncInterval;
 		bool mWBuffer;
 
 		size_t mBatchCount;
@@ -1395,9 +1559,19 @@ namespace Ogre
 		float mDerivedDepthBiasMultiplier;
 		float mDerivedDepthBiasSlopeScale;
 
+        /// a global vertex buffer for global instancing
+        HardwareVertexBufferSharedPtr mGlobalInstanceVertexBuffer;
+        /// a vertex declaration for the global vertex buffer for the global instancing
+        VertexDeclaration* mGlobalInstanceVertexBufferVertexDeclaration;
+        /// the number of global instances (this number will be multiply by the render op instance number) 
+        size_t mGlobalNumberOfInstances;
+
+		/// is fixed pipeline enabled
+		bool mEnableFixedPipeline;
+
 		/** updates pass iteration rendering state including bound gpu program parameter
 		pass iteration auto constant entry
-		@returns True if more iterations are required
+		@return True if more iterations are required
 		*/
 		bool updatePassIterationRenderState(void);
 
@@ -1416,6 +1590,9 @@ namespace Ogre
 		bool mVertexProgramBound;
 		bool mGeometryProgramBound;
 		bool mFragmentProgramBound;
+		bool mTesselationHullProgramBound;
+		bool mTesselationDomainProgramBound;
+		bool mComputeProgramBound;
 
 		// Recording user clip planes
 		PlaneList mClipPlanes;
@@ -1435,6 +1612,7 @@ namespace Ogre
 
 
 		DriverVersion mDriverVersion;
+        uint16 mNativeShadingLanguageVersion;
 
 		bool mTexProjRelative;
 		Vector3 mTexProjRelativeOrigin;
@@ -1446,4 +1624,6 @@ namespace Ogre
 	/** @} */
 }
 
+#include "OgreHeaderSuffix.h"
+
 #endif
