@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -39,46 +39,47 @@ THE SOFTWARE.
 #include "OgreVector4.h"
 #include "OgreException.h"
 #include "OgreUserObjectBindings.h"
+#include "OgreHeaderPrefix.h"
 
 namespace Ogre {
 
-	/** \addtogroup Core
-	*  @{
-	*/
-	/** \addtogroup Scene
-	*  @{
-	*/
-	/** Abstract class defining the interface all renderable objects must implement.
-        @remarks
-            This interface abstracts renderable discrete objects which will be queued in the render pipeline,
-            grouped by material. Classes implementing this interface must be based on a single material, a single
-            world matrix (or a collection of world matrices which are blended by weights), and must be 
-            renderable via a single render operation.
-        @par
-            Note that deciding whether to put these objects in the rendering pipeline is done from the more specific
-            classes e.g. entities. Only once it is decided that the specific class is to be rendered is the abstract version
-            created (could be more than one per visible object) and pushed onto the rendering queue.
+    /** \addtogroup Core
+    *  @{
+    */
+    /** \addtogroup Scene
+    *  @{
+    */
+    /** Abstract class defining the interface all renderable objects must implement.
+    @remarks
+        This interface abstracts renderable discrete objects which will be queued in the render pipeline,
+        grouped by material. Classes implementing this interface must be based on a single material, a single
+        world matrix (or a collection of world matrices which are blended by weights), and must be 
+        renderable via a single render operation.
+    @par
+        Note that deciding whether to put these objects in the rendering pipeline is done from the more specific
+        classes e.g. entities. Only once it is decided that the specific class is to be rendered is the abstract version
+        created (could be more than one per visible object) and pushed onto the rendering queue.
     */
     class _OgreExport Renderable
     {
-	public:
-		/** An internal class that should be used only by a render system for internal use 
-		@remarks
-		This class was created so a render system can associate internal data to this class.
-		The need for this class started when the DX10 render system needed to save state objects.
-		*/
-		class RenderSystemData {}; 
     public:
-		Renderable() : mPolygonModeOverrideable(true), mUseIdentityProjection(false), mUseIdentityView(false), mRenderSystemData(NULL) {}
+        /** An internal class that should be used only by a render system for internal use 
+        @remarks
+            This class was created so a render system can associate internal data to this class.
+            The need for this class started when the DX10 render system needed to save state objects.
+        */
+        class RenderSystemData {}; 
+    public:
+        Renderable() : mPolygonModeOverrideable(true), mUseIdentityProjection(false), mUseIdentityView(false), mRenderSystemData(NULL) {}
         /** Virtual destructor needed as class has virtual methods. */
         virtual ~Renderable() 
-		{
-			if (mRenderSystemData)
-			{
-				delete mRenderSystemData;
-				mRenderSystemData = NULL;
-			}			
-		}
+        {
+            if (mRenderSystemData)
+            {
+                delete mRenderSystemData;
+                mRenderSystemData = NULL;
+            }
+        }
         /** Retrieves a weak reference to the material this renderable object uses.
         @remarks
             Note that the Renderable also has the option to override the getTechnique method
@@ -95,49 +96,49 @@ namespace Ogre {
         */
         virtual void getRenderOperation(RenderOperation& op) = 0;
 
-		/** Called just prior to the Renderable being rendered. 
-		@remarks
-			OGRE is a queued renderer, so the actual render commands are executed 
-			at a later time than the point at which an object is discovered to be
-			visible. This allows ordering & grouping of renders without the discovery
-			process having to be aware of it. It also means OGRE uses declarative
-			render information rather than immediate mode rendering - this is very useful
-			in that certain effects and processes can automatically be applied to 
-			a wide range of scenes, but the downside is that special cases are
-			more difficult to handle, because there is not the declared state to 
-			cope with it. 
-		@par
-			This method allows a Renderable to do something special at the actual
-			point of rendering if it wishes to. When this method is called, all the
-			material render state as declared by this Renderable has already been set, 
-			all that is left to do is to bind the buffers and perform the render. 
-			The Renderable may modify render state itself if it wants to (and restore it in the 
-			postRender call) before the automated render happens, or by returning
-			'false' from this method can actually suppress the automatic render
-			and perform one of its own.
-		@return
-			true if the automatic render should proceed, false to skip it on 
-			the assumption that the Renderable has done it manually.
-		*/
-		virtual bool preRender(SceneManager* sm, RenderSystem* rsys)
+        /** Called just prior to the Renderable being rendered. 
+        @remarks
+            OGRE is a queued renderer, so the actual render commands are executed 
+            at a later time than the point at which an object is discovered to be
+            visible. This allows ordering & grouping of renders without the discovery
+            process having to be aware of it. It also means OGRE uses declarative
+            render information rather than immediate mode rendering - this is very useful
+            in that certain effects and processes can automatically be applied to 
+            a wide range of scenes, but the downside is that special cases are
+            more difficult to handle, because there is not the declared state to 
+            cope with it. 
+        @par
+            This method allows a Renderable to do something special at the actual
+            point of rendering if it wishes to. When this method is called, all the
+            material render state as declared by this Renderable has already been set, 
+            all that is left to do is to bind the buffers and perform the render. 
+            The Renderable may modify render state itself if it wants to (and restore it in the 
+            postRender call) before the automated render happens, or by returning
+            'false' from this method can actually suppress the automatic render
+            and perform one of its own.
+        @return
+            true if the automatic render should proceed, false to skip it on 
+            the assumption that the Renderable has done it manually.
+        */
+        virtual bool preRender(SceneManager* sm, RenderSystem* rsys)
                 { (void)sm; (void)rsys; return true; }
 
-		/** Called immediately after the Renderable has been rendered. 
-		*/
-		virtual void postRender(SceneManager* sm, RenderSystem* rsys)
+        /** Called immediately after the Renderable has been rendered. 
+        */
+        virtual void postRender(SceneManager* sm, RenderSystem* rsys)
                 { (void)sm; (void)rsys; }
 
         /** Gets the world transform matrix / matrices for this renderable object.
-            @remarks
-                If the object has any derived transforms, these are expected to be up to date as long as
-                all the SceneNode structures have been updated before this is called.
-            @par
-                This method will populate xform with 1 matrix if it does not use vertex blending. If it
-                does use vertex blending it will fill the passed in pointer with an array of matrices,
-                the length being the value returned from getNumWorldTransforms.
-            @note
-                Internal Ogre never supports non-affine matrix for world transform matrix/matrices,
-                the behavior is undefined if returns non-affine matrix here. @see Matrix4::isAffine.
+        @remarks
+            If the object has any derived transforms, these are expected to be up to date as long as
+            all the SceneNode structures have been updated before this is called.
+        @par
+            This method will populate transform with 1 matrix if it does not use vertex blending. If it
+            does use vertex blending it will fill the passed in pointer with an array of matrices,
+            the length being the value returned from getNumWorldTransforms.
+        @note
+            Internal Ogre never supports non-affine matrix for world transform matrix/matrices,
+            the behavior is undefined if returns non-affine matrix here. @see Matrix4::isAffine.
         */
         virtual void getWorldTransforms(Matrix4* xform) const = 0;
 
@@ -201,12 +202,12 @@ namespace Ogre {
         */
         bool getUseIdentityView(void) const { return mUseIdentityView; }
 
-		/** Returns the camera-relative squared depth of this renderable.
-		@remarks
-			Used to sort transparent objects. Squared depth is used rather than
-			actual depth to avoid having to perform a square root on the result.
-		*/
-		virtual Real getSquaredViewDepth(const Camera* cam) const = 0;
+        /** Returns the camera-relative squared depth of this renderable.
+        @remarks
+            Used to sort transparent objects. Squared depth is used rather than
+            actual depth to avoid having to perform a square root on the result.
+        */
+        virtual Real getSquaredViewDepth(const Camera* cam) const = 0;
 
         /** Gets a list of lights, ordered relative to how close they are to this renderable.
         @remarks
@@ -242,8 +243,26 @@ namespace Ogre {
             mCustomParameters[index] = value;
         }
 
+        /** Removes a custom value which is associated with this Renderable at the given index.
+        @param index Index of the parameter to remove.
+            @see setCustomParameter for full details.
+        */
+        void removeCustomParameter(size_t index)
+        {
+            mCustomParameters.erase(index);
+        }
+
+        /** Checks whether a custom value is associated with this Renderable at the given index.
+        @param index Index of the parameter to check for existence.
+            @see setCustomParameter for full details.
+        */
+        bool hasCustomParameter(size_t index) const
+        {
+            return mCustomParameters.find(index) != mCustomParameters.end();
+        }
+
         /** Gets the custom value associated with this Renderable at the given index.
-        @param
+        @param index Index of the parameter to retrieve.
             @see setCustomParameter for full details.
         */
         const Vector4& getCustomParameter(size_t index) const
@@ -293,118 +312,119 @@ namespace Ogre {
             if (i != mCustomParameters.end())
             {
                 params->_writeRawConstant(constantEntry.physicalIndex, i->second, 
-					constantEntry.elementCount);
+                    constantEntry.elementCount);
             }
         }
 
-		/** Sets whether this renderable's chosen detail level can be
-			overridden (downgraded) by the camera setting. 
-		@param override true means that a lower camera detail will override this
-			renderables detail level, false means it won't.
-		*/
-		virtual void setPolygonModeOverrideable(bool override)
-		{
-			mPolygonModeOverrideable = override;
-		}
+        /** Sets whether this renderable's chosen detail level can be
+            overridden (downgraded) by the camera setting. 
+        @param override true means that a lower camera detail will override this
+            renderables detail level, false means it won't.
+        */
+        virtual void setPolygonModeOverrideable(bool override)
+        {
+            mPolygonModeOverrideable = override;
+        }
 
-		/** Gets whether this renderable's chosen detail level can be
-			overridden (downgraded) by the camera setting. 
-		*/
-		virtual bool getPolygonModeOverrideable(void) const
-		{
-			return mPolygonModeOverrideable;
-		}
+        /** Gets whether this renderable's chosen detail level can be
+            overridden (downgraded) by the camera setting. 
+        */
+        virtual bool getPolygonModeOverrideable(void) const
+        {
+            return mPolygonModeOverrideable;
+        }
 
-		/** @deprecated use UserObjectBindings::setUserAny via getUserObjectBindings() instead.
-			Sets any kind of user value on this object.
-		@remarks
-			This method allows you to associate any user value you like with 
-			this Renderable. This can be a pointer back to one of your own
-			classes for instance.
-		*/
-		virtual void setUserAny(const Any& anything) { getUserObjectBindings().setUserAny(anything); }
+        /** @deprecated use UserObjectBindings::setUserAny via getUserObjectBindings() instead.
+            Sets any kind of user value on this object.
+        @remarks
+            This method allows you to associate any user value you like with 
+            this Renderable. This can be a pointer back to one of your own
+            classes for instance.
+        */
+        OGRE_DEPRECATED virtual void setUserAny(const Any& anything) { getUserObjectBindings().setUserAny(anything); }
 
-		/** @deprecated use UserObjectBindings::getUserAny via getUserObjectBindings() instead.
-			Retrieves the custom user value associated with this object.
-		*/
-		virtual const Any& getUserAny(void) const { return getUserObjectBindings().getUserAny(); }
+        /** @deprecated use UserObjectBindings::getUserAny via getUserObjectBindings() instead.
+            Retrieves the custom user value associated with this object.
+        */
+        OGRE_DEPRECATED virtual const Any& getUserAny(void) const { return getUserObjectBindings().getUserAny(); }
 
-		/** Return an instance of user objects binding associated with this class.
-		You can use it to associate one or more custom objects with this class instance.
-		@see UserObjectBindings::setUserAny.
-		*/
-		UserObjectBindings&	getUserObjectBindings() { return mUserObjectBindings; }
+        /** Return an instance of user objects binding associated with this class.
+            You can use it to associate one or more custom objects with this class instance.
+        @see UserObjectBindings::setUserAny.
+        */
+        UserObjectBindings&	getUserObjectBindings() { return mUserObjectBindings; }
 
-		/** Return an instance of user objects binding associated with this class.
-		You can use it to associate one or more custom objects with this class instance.
-		@see UserObjectBindings::setUserAny.		
-		*/
-		const UserObjectBindings& getUserObjectBindings() const { return mUserObjectBindings; }
+        /** Return an instance of user objects binding associated with this class.
+            You can use it to associate one or more custom objects with this class instance.
+        @see UserObjectBindings::setUserAny.
+        */
+        const UserObjectBindings& getUserObjectBindings() const { return mUserObjectBindings; }
 
 
-		/** Visitor object that can be used to iterate over a collection of Renderable
-		instances abstractly.
-		@remarks
-		Different scene objects use Renderable differently; some will have a 
-		single Renderable, others will have many. This visitor interface allows
-		classes using Renderable to expose a clean way for external code to
-		get access to the contained Renderable instance(s) that it will
-		eventually add to the render queue.
-		@par
-		To actually have this method called, you have to call a method on the
-		class containing the Renderable instances. One example is 
-		MovableObject::visitRenderables.
-		*/
-		class Visitor
-		{
-		public:
+        /** Visitor object that can be used to iterate over a collection of Renderable
+            instances abstractly.
+        @remarks
+            Different scene objects use Renderable differently; some will have a 
+            single Renderable, others will have many. This visitor interface allows
+            classes using Renderable to expose a clean way for external code to
+            get access to the contained Renderable instance(s) that it will
+            eventually add to the render queue.
+        @par
+            To actually have this method called, you have to call a method on the
+            class containing the Renderable instances. One example is 
+            MovableObject::visitRenderables.
+        */
+        class Visitor
+        {
+        public:
             /** Virtual destructor needed as class has virtual methods. */
             virtual ~Visitor() { }
-			/** Generic visitor method. 
-			@param rend The Renderable instance being visited
-			@param lodIndex The LOD index to which this Renderable belongs. Some
-				objects support LOD and this will tell you whether the Renderable
-				you're looking at is from the top LOD (0) or otherwise
-			@param isDebug Whether this is a debug renderable or not.
-			@param pAny Optional pointer to some additional data that the class
-				calling the visitor may populate if it chooses to.
-			*/
-			virtual void visit(Renderable* rend, ushort lodIndex, bool isDebug, 
-				Any* pAny = 0) = 0;
-		};
+            /** Generic visitor method. 
+            @param rend The Renderable instance being visited
+            @param lodIndex The LOD index to which this Renderable belongs. Some
+                objects support LOD and this will tell you whether the Renderable
+                you're looking at is from the top LOD (0) or otherwise
+            @param isDebug Whether this is a debug renderable or not.
+            @param pAny Optional pointer to some additional data that the class
+                calling the visitor may populate if it chooses to.
+            */
+            virtual void visit(Renderable* rend, ushort lodIndex, bool isDebug, 
+                Any* pAny = 0) = 0;
+        };
 
-		/** Sets render system private data
-		@remarks
-		This should only be used by a render system
-		*/
-		virtual RenderSystemData * getRenderSystemData() const 
-		{ 
-			return mRenderSystemData; 
-		}
-		/** gets render system private data
-		@remarks
-		This should only be used by a render system
-		*/
-		virtual void setRenderSystemData(RenderSystemData * val) const
-		{ 
-			mRenderSystemData = val; 
-		}
+        /** Gets RenderSystem private data
+        @remarks
+            This should only be used by a RenderSystem
+        */
+        virtual RenderSystemData * getRenderSystemData() const 
+        { 
+            return mRenderSystemData; 
+        }
+        /** Sets RenderSystem private data
+        @remarks
+            This should only be used by a RenderSystem
+        */
+        virtual void setRenderSystemData(RenderSystemData * val) const
+        { 
+            mRenderSystemData = val; 
+        }
 
 
     protected:
         typedef map<size_t, Vector4>::type CustomParameterMap;
         CustomParameterMap mCustomParameters;
-		bool mPolygonModeOverrideable;
+        bool mPolygonModeOverrideable;
         bool mUseIdentityProjection;
         bool mUseIdentityView;
-		UserObjectBindings mUserObjectBindings;		 // User objects binding.
-		mutable RenderSystemData * mRenderSystemData;// this should be used only by a render system for internal use
-	};
+        UserObjectBindings mUserObjectBindings;      /// User objects binding.
+        mutable RenderSystemData * mRenderSystemData;/// This should be used only by a render system for internal use
+    };
 
-	/** @} */
-	/** @} */
+    /** @} */
+    /** @} */
 
+} // namespace Ogre
 
-}
+#include "OgreHeaderSuffix.h"
 
 #endif //__Renderable_H__
