@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -37,9 +37,58 @@ THE SOFTWARE.
 #include "OgreDataStream.h"
 #include "OgreStringVector.h"
 #include "OgreScriptLoader.h"
+#include "OgreHeaderPrefix.h"
 
 namespace Ogre {
 
+    /** Template class describing a simple pool of items.
+     */
+	template <typename T>
+	class Pool
+	{
+	protected:
+		typedef typename list<T>::type ItemList;
+		ItemList mItems;
+		OGRE_AUTO_MUTEX;
+	public:
+		Pool() {}
+		virtual ~Pool() {}
+
+		/** Get the next item from the pool.
+         @return pair indicating whether there was a free item, and the item if so
+         */
+		virtual std::pair<bool, T> removeItem()
+		{
+            OGRE_LOCK_AUTO_MUTEX;
+			std::pair<bool, T> ret;
+			if (mItems.empty())
+			{
+				ret.first = false;
+			}
+			else
+			{
+				ret.first = true;
+				ret.second = mItems.front();
+				mItems.pop_front();
+			}
+			return ret;
+		}
+
+		/** Add a new item to the pool.
+         */
+		virtual void addItem(const T& i)
+		{
+            OGRE_LOCK_AUTO_MUTEX;
+			mItems.push_front(i);
+		}
+		/// Clear the pool
+		virtual void clear()
+		{
+            OGRE_LOCK_AUTO_MUTEX;
+			mItems.clear();
+		}
+	};
+
 	/** \addtogroup Core
 	*  @{
 	*/
@@ -73,7 +122,7 @@ namespace Ogre {
 	class _OgreExport ResourceManager : public ScriptLoader, public ResourceAlloc
     {
     public:
-		OGRE_AUTO_MUTEX // public to allow external locking
+        OGRE_AUTO_MUTEX; // public to allow external locking
         ResourceManager();
         virtual ~ResourceManager();
 
@@ -96,7 +145,7 @@ namespace Ogre {
         @param createParams If any parameters are required to create an instance,
             they should be supplied here as name / value pairs
         */
-        virtual ResourcePtr create(const String& name, const String& group, 
+        virtual ResourcePtr createResource(const String& name, const String& group,
             bool isManual = false, ManualResourceLoader* loader = 0, 
             const NameValuePairList* createParams = 0);
 
@@ -109,9 +158,9 @@ namespace Ogre {
 			in one call so there are no race conditions if using multiple
 			threads that could cause getByName() to return null, but create() to
 			fail because another thread created a resource in between.
-		@see ResourceManager::create
-		@see ResourceManager::getByName
-		@returns A pair, the first element being the pointer, and the second being 
+		@see ResourceManager::createResource
+		@see ResourceManager::getResourceByName
+		@return A pair, the first element being the pointer, and the second being 
 			an indicator specifying whether the resource was newly created.
 		*/
 		virtual ResourceCreateOrRetrieveResult createOrRetrieve(const String& name, 
@@ -126,14 +175,14 @@ namespace Ogre {
                 is not permanent and the Resource is not destroyed; it simply needs to be reloaded when
                 next used.
         */
-        virtual void setMemoryBudget( size_t bytes);
+        virtual void setMemoryBudget(size_t bytes);
 
         /** Get the limit on the amount of memory this resource handler may use.
         */
         virtual size_t getMemoryBudget(void) const;
 
 		/** Gets the current memory usage, in bytes. */
-		virtual size_t getMemoryUsage(void) const { return mMemoryUsage; }
+		virtual size_t getMemoryUsage(void) const { return mMemoryUsage.get(); }
 
 		/** Unloads a single resource by name.
 		@remarks
@@ -299,7 +348,7 @@ namespace Ogre {
 
         /** Retrieves a pointer to a resource by name, or null if the resource does not exist.
         */
-        virtual ResourcePtr getByName(const String& name, const String& groupName = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME);
+        virtual ResourcePtr getResourceByName(const String& name, const String& groupName = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME);
         /** Retrieves a pointer to a resource by handle, or null if the resource does not exist.
         */
         virtual ResourcePtr getByHandle(ResourceHandle handle);
@@ -307,7 +356,7 @@ namespace Ogre {
 		/// Returns whether the named resource exists in this manager
 		virtual bool resourceExists(const String& name)
 		{
-			return !getByName(name).isNull();
+			return !getResourceByName(name).isNull();
 		}
 		/// Returns whether a resource with the given handle exists in this manager
 		virtual bool resourceExists(ResourceHandle handle)
@@ -381,7 +430,7 @@ namespace Ogre {
 		@par
 			This method lets you determine the file pattern which will be used
 			to identify scripts intended for this manager.
-		@returns
+		@return
 			A list of file patterns, in the order they should be searched in.
 		@see isScriptingSupported, parseScript
 		*/
@@ -483,7 +532,7 @@ namespace Ogre {
 		virtual void addImpl( ResourcePtr& res );
 		/** Remove a resource from this manager; remove it from the lists. */
 		virtual void removeImpl( ResourcePtr& res );
-		/** Checks memory usage and pages out if required.
+		/** Checks memory usage and pages out if required. This is automatically done after a new resource is loaded.
 		*/
 		virtual void checkUsage(void);
 
@@ -496,9 +545,9 @@ namespace Ogre {
         ResourceHandleMap mResourcesByHandle;
         ResourceMap mResources;
 		ResourceWithGroupMap mResourcesWithGroup;
-        ResourceHandle mNextHandle;
-        size_t mMemoryBudget; // In bytes
-        size_t mMemoryUsage; // In bytes
+        size_t mMemoryBudget; /// In bytes
+        AtomicScalar<ResourceHandle> mNextHandle;
+        AtomicScalar<size_t> mMemoryUsage; /// In bytes
 
         bool mVerbose;
 
@@ -525,14 +574,13 @@ namespace Ogre {
 	protected:
 		typedef map<String, ResourcePool*>::type ResourcePoolMap;
 		ResourcePoolMap mResourcePoolMap;
-
-
-    
-
     };
+
 	/** @} */
 	/** @} */
 
 }
 
+#include "OgreHeaderSuffix.h"
+
 #endif
