@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -42,13 +42,13 @@ THE SOFTWARE.
 
 namespace Ogre {
 
-	/** \addtogroup Core
-	*  @{
-	*/
-	/** \addtogroup General
-	*  @{
-	*/
-	/** Class for converting the core Ogre data types to/from Strings.
+    /** \addtogroup Core
+    *  @{
+    */
+    /** \addtogroup General
+    *  @{
+    */
+    /** Class for converting the core Ogre data types to/from Strings.
     @remarks
         The code for converting values to and from strings is here as a separate
         class to avoid coupling String to other datatypes (and vice-versa) which reduces
@@ -72,60 +72,70 @@ namespace Ogre {
         /** Converts a Real to a String. */
         static String toString(Real val, unsigned short precision = 6, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
+#if OGRE_DOUBLE_PRECISION == 1
+        /** Converts a float to a String. */
+        static String toString(float val, unsigned short precision = 6,
+                               unsigned short width = 0, char fill = ' ',
+                               std::ios::fmtflags flags = std::ios::fmtflags(0));
+#else
+        /** Converts a double to a String. */
+        static String toString(double val, unsigned short precision = 6,
+                               unsigned short width = 0, char fill = ' ',
+                               std::ios::fmtflags flags = std::ios::fmtflags(0));
+#endif
         /** Converts a Radian to a String. */
         static String toString(Radian val, unsigned short precision = 6, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) )
+            std::ios::fmtflags flags = std::ios::fmtflags(0))
         {
             return toString(val.valueAngleUnits(), precision, width, fill, flags);
         }
         /** Converts a Degree to a String. */
         static String toString(Degree val, unsigned short precision = 6, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) )
+            std::ios::fmtflags flags = std::ios::fmtflags(0))
         {
             return toString(val.valueAngleUnits(), precision, width, fill, flags);
         }
         /** Converts an int to a String. */
         static String toString(int val, unsigned short width = 0, 
             char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
 #if OGRE_PLATFORM != OGRE_PLATFORM_NACL &&  ( OGRE_ARCH_TYPE == OGRE_ARCHITECTURE_64 || OGRE_PLATFORM == OGRE_PLATFORM_APPLE || OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS || defined(__OpenBSD__) )
         /** Converts an unsigned int to a String. */
         static String toString(unsigned int val, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
         /** Converts a size_t to a String. */
         static String toString(size_t val, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
         #if OGRE_COMPILER == OGRE_COMPILER_MSVC
-                /** Converts an unsigned long to a String. */
-                static String toString(unsigned long val, 
-                    unsigned short width = 0, char fill = ' ', 
-                    std::ios::fmtflags flags = std::ios::fmtflags(0) );
-
+        /** Converts an unsigned long to a String. */
+        static String toString(unsigned long val, 
+            unsigned short width = 0, char fill = ' ', 
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
         #endif
 #else
         /** Converts a size_t to a String. */
         static String toString(size_t val, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
         /** Converts an unsigned long to a String. */
         static String toString(unsigned long val, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
 #endif
         /** Converts a long to a String. */
         static String toString(long val, 
             unsigned short width = 0, char fill = ' ', 
-            std::ios::fmtflags flags = std::ios::fmtflags(0) );
+            std::ios::fmtflags flags = std::ios::fmtflags(0));
         /** Converts a boolean to a String. 
         @param yesNo If set to true, result is 'yes' or 'no' instead of 'true' or 'false'
         */
         static String toString(bool val, bool yesNo = false);
-		/** Converts a Vector2 to a String. 
+        /** Converts a Vector2 to a String. 
         @remarks
             Format is "x y" (i.e. 2x Real values, space delimited)
         */
@@ -135,7 +145,7 @@ namespace Ogre {
             Format is "x y z" (i.e. 3x Real values, space delimited)
         */
         static String toString(const Vector3& val);
-		/** Converts a Vector4 to a String. 
+        /** Converts a Vector4 to a String. 
         @remarks
             Format is "x y z w" (i.e. 4x Real values, space delimited)
         */
@@ -178,8 +188,8 @@ namespace Ogre {
             0.0 if the value could not be parsed, otherwise the Angle version of the String.
         */
         static inline Radian parseAngle(const String& val, Radian defaultValue = Radian(0)) {
-			return Angle(parseReal(val, defaultValue.valueRadians()));
-		}
+            return Angle(parseReal(val, defaultValue.valueRadians()));
+        }
         /** Converts a String to a whole number. 
         @return
             0.0 if the value could not be parsed, otherwise the numeric version of the String.
@@ -208,16 +218,16 @@ namespace Ogre {
         /** Converts a String to a boolean. 
         @remarks
             Returns true if case-insensitive match of the start of the string
-			matches "true", "yes" or "1", false otherwise.
+            matches "true", "yes" or "1", false otherwise.
         */
         static bool parseBool(const String& val, bool defaultValue = 0);
-		/** Parses a Vector2 out of a String.
+        /** Parses a Vector2 out of a String.
         @remarks
             Format is "x y" ie. 2 Real components, space delimited. Failure to parse returns
             Vector2::ZERO.
         */
-		static Vector2 parseVector2(const String& val, const Vector2& defaultValue = Vector2::ZERO);
-		/** Parses a Vector3 out of a String.
+        static Vector2 parseVector2(const String& val, const Vector2& defaultValue = Vector2::ZERO);
+        /** Parses a Vector3 out of a String.
         @remarks
             Format is "x y z" ie. 3 Real components, space delimited. Failure to parse returns
             Vector3::ZERO.
@@ -254,7 +264,7 @@ namespace Ogre {
         */
         static ColourValue parseColourValue(const String& val, const ColourValue& defaultValue = ColourValue::Black);
 
-        /** Pareses a StringVector from a string.
+        /** Parses a StringVector from a string.
         @remarks
             Strings must not contain spaces since space is used as a delimiter in
             the output.
@@ -262,10 +272,29 @@ namespace Ogre {
         static StringVector parseStringVector(const String& val);
         /** Checks the String is a valid number value. */
         static bool isNumber(const String& val);
+
+        //-----------------------------------------------------------------------
+        static void setDefaultStringLocale(String loc)
+        {
+            msDefaultStringLocale = loc;
+            msLocale = std::locale(msDefaultStringLocale.c_str());
+        }
+        //-----------------------------------------------------------------------
+        static String getDefaultStringLocale(void) { return msDefaultStringLocale; }
+        //-----------------------------------------------------------------------
+        static void setUseLocale(bool useLocale) { msUseLocale = useLocale; }
+        //-----------------------------------------------------------------------
+        static bool isUseLocale() { return msUseLocale; }
+        //-----------------------------------------------------------------------
+
+    protected:
+        static String msDefaultStringLocale;
+        static std::locale msLocale;
+        static bool msUseLocale;
     };
 
-	/** @} */
-	/** @} */
+    /** @} */
+    /** @} */
 
 }
 
