@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,9 +29,10 @@ THE SOFTWARE.
 #define __OgreWorkQueue_H__
 
 #include "OgrePrerequisites.h"
-#include "OgreAtomicWrappers.h"
 #include "OgreAny.h"
 #include "OgreSharedPtr.h"
+#include "Threading/OgreThreadHeaders.h"
+#include "OgreHeaderPrefix.h"
 
 namespace Ogre
 {
@@ -69,10 +70,10 @@ namespace Ogre
 	class _OgreExport WorkQueue : public UtilityAlloc
 	{
 	protected:
-		typedef std::map<String, uint16> ChannelMap;
+		typedef map<String, uint16>::type ChannelMap;
 		ChannelMap mChannelMap;
 		uint16 mNextChannel;
-		OGRE_MUTEX(mChannelMapMutex)
+		OGRE_MUTEX(mChannelMapMutex);
 	public:
 		/// Numeric identifier for a request
 		typedef unsigned long long int RequestID;
@@ -261,10 +262,15 @@ namespace Ogre
 			if it fails.
 		@param forceSynchronous Forces the request to be processed immediately
 			even if threading is enabled.
-		@returns The ID of the request that has been added
+		@param idleThread Request should be processed on the idle thread.
+		    Idle requests will be processed on a single worker thread. You should use this in the following situations:
+			1. If a request handler can't process multiple requests in parallel.
+			2. If you add lot of requests, but you want to keep the game fast.
+			3. If you have lot of more important threads. (example: physics).
+		@return The ID of the request that has been added
 		*/
 		virtual RequestID addRequest(uint16 channel, uint16 requestType, const Any& rData, uint8 retryCount = 0, 
-			bool forceSynchronous = false) = 0;
+			bool forceSynchronous = false, bool idleThread = false) = 0;
 
 		/** Abort a previously issued request.
 		If the request is still waiting to be processed, it will be 
@@ -276,10 +282,19 @@ namespace Ogre
 		/** Abort all previously issued requests in a given channel.
 		Any requests still waiting to be processed of the given channel, will be 
 		removed from the queue.
+		Requests which are processed, but response handler is not called will also be removed.
 		@param channel The type of request to be aborted
 		*/
 		virtual void abortRequestsByChannel(uint16 channel) = 0;
 
+		/** Abort all previously issued requests in a given channel.
+		Any requests still waiting to be processed of the given channel, will be 
+		removed from the queue.
+		It will not remove requests, where the request handler is already called.
+		@param channel The type of request to be aborted
+		*/
+		virtual void abortPendingRequestsByChannel(uint16 channel) = 0;
+
 		/** Abort all previously issued requests.
 		Any requests still waiting to be processed will be removed from the queue.
 		Any requests that are being processed will still complete.
@@ -417,11 +432,13 @@ namespace Ogre
 
 		/// @copydoc WorkQueue::addRequest
 		virtual RequestID addRequest(uint16 channel, uint16 requestType, const Any& rData, uint8 retryCount = 0, 
-			bool forceSynchronous = false);
+			bool forceSynchronous = false, bool idleThread = false);
 		/// @copydoc WorkQueue::abortRequest
 		virtual void abortRequest(RequestID id);
 		/// @copydoc WorkQueue::abortRequestsByChannel
 		virtual void abortRequestsByChannel(uint16 channel);
+		/// @copydoc WorkQueue::abortPendingRequestsByChannel
+		virtual void abortPendingRequestsByChannel(uint16 channel);
 		/// @copydoc WorkQueue::abortAllRequests
 		virtual void abortAllRequests();
 		/// @copydoc WorkQueue::setPaused
@@ -447,12 +464,12 @@ namespace Ogre
 
 		typedef deque<Request*>::type RequestQueue;
 		typedef deque<Response*>::type ResponseQueue;
-		RequestQueue mRequestQueue;
-		RequestQueue mProcessQueue;
-		ResponseQueue mResponseQueue;
+		RequestQueue mRequestQueue; // Guarded by mRequestMutex
+		RequestQueue mProcessQueue; // Guarded by mProcessMutex
+		ResponseQueue mResponseQueue; // Guarded by mResponseMutex
 
 		/// Thread function
-		struct WorkerFunc OGRE_THREAD_WORKER_INHERIT
+		struct _OgreExport WorkerFunc OGRE_THREAD_WORKER_INHERIT
 		{
 			DefaultWorkQueueBase* mQueue;
 
@@ -460,6 +477,8 @@ namespace Ogre
 				: mQueue(q) {}
 
 			void operator()();
+			
+			void operator()() const;
 
 			void run();
 		};
@@ -521,14 +540,19 @@ namespace Ogre
 
 		RequestHandlerListByChannel mRequestHandlers;
 		ResponseHandlerListByChannel mResponseHandlers;
-		RequestID mRequestCount;
+		RequestID mRequestCount; // Guarded by mRequestMutex
 		bool mPaused;
 		bool mAcceptRequests;
 		bool mShuttingDown;
 
-		OGRE_MUTEX(mRequestMutex)
-		OGRE_MUTEX(mProcessMutex)
-		OGRE_MUTEX(mResponseMutex)
+		//NOTE: If you lock multiple mutexes at the same time, the order is important!
+		// For example if threadA locks mIdleMutex first then tries to lock mProcessMutex,
+		// and threadB locks mProcessMutex first, then mIdleMutex. In this case you can get livelock and the system is dead!
+		//RULE: Lock mProcessMutex before other mutex, to prevent livelocks
+		OGRE_MUTEX(mIdleMutex);
+		OGRE_MUTEX(mRequestMutex);
+		OGRE_MUTEX(mProcessMutex);
+		OGRE_MUTEX(mResponseMutex);
 		OGRE_RW_MUTEX(mRequestHandlerMutex);
 
 
@@ -539,7 +563,13 @@ namespace Ogre
 		virtual void notifyWorkers() = 0;
 		/// Put a Request on the queue with a specific RequestID.
 		void addRequestWithRID(RequestID rid, uint16 channel, uint16 requestType, const Any& rData, uint8 retryCount);
+		
+		RequestQueue mIdleRequestQueue; // Guarded by mIdleMutex
+		bool mIdleThreadRunning; // Guarded by mIdleMutex
+		Request* mIdleProcessed; // Guarded by mProcessMutex
+		
 
+		bool processIdleRequests();
 	};
 
 
@@ -551,6 +581,7 @@ namespace Ogre
 
 }
 
+#include "OgreHeaderSuffix.h"
 
 #endif
 
