@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,7 @@ THE SOFTWARE.
 #include <stdlib.h>
 
 namespace Ogre {
+    namespace GLSL {
 
 /**
  * This is a simplistic C/C++-like preprocessor.
@@ -90,7 +91,7 @@ class CPreprocessor
             TK_STRING,       // A string
             TK_COMMENT,      // A block comment
             TK_LINECOMMENT,  // A line comment
-            TK_TEXT,         // An unparsed text (cannot be returned from GetToken())
+            TK_TEXT          // An unparsed text (cannot be returned from GetToken())
         };
 
         /// Token type
@@ -519,6 +520,7 @@ public:
     void *ErrorData;
 };
 
+} // namespace GLSL
 } // namespace Ogre
 
 #endif // __OGRE_CPREPROCESSOR_H__
