@@ -4,7 +4,7 @@
  (Object-oriented Graphics Rendering Engine)
  For the latest info, see http://www.ogre3d.org/
  
- Copyright (c) 2000-2011 Torus Knot Software Ltd
+ Copyright (c) 2000-2013 Torus Knot Software Ltd
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -32,7 +32,7 @@
 #include "SdkTrays.h"
 #include "SdkCameraMan.h"
 
-#ifdef USE_RTSHADER_SYSTEM
+#ifdef INCLUDE_RTSHADER_SYSTEM
 #include "OgreRTShaderSystem.h"
 #endif
 
@@ -43,8 +43,7 @@ namespace OgreBites
 	=============================================================================*/
 	class SdkSample : public Sample, public SdkTrayListener
     {
-    public:
-
+	public:
 		SdkSample()
 		{
 			// so we don't have to worry about checking if these keys exist later
@@ -116,7 +115,7 @@ namespace OgreBites
 					mDetailsPanel->setParamValue(6, Ogre::StringConverter::toString(mCamera->getDerivedOrientation().y));
 					mDetailsPanel->setParamValue(7, Ogre::StringConverter::toString(mCamera->getDerivedOrientation().z));
 
-#ifdef USE_RTSHADER_SYSTEM
+#ifdef INCLUDE_RTSHADER_SYSTEM
 					mDetailsPanel->setParamValue(14, Ogre::StringConverter::toString(mShaderGenerator->getVertexShaderCount()));
 					mDetailsPanel->setParamValue(15, Ogre::StringConverter::toString(mShaderGenerator->getFragmentShaderCount()));		
 #endif
@@ -158,13 +157,13 @@ namespace OgreBites
 					mDetailsPanel->hide();
 				}
 			}
-			else if (evt.key == OIS::KC_T)   // cycle polygon rendering mode
+			else if (evt.key == OIS::KC_T)   // cycle texture filtering mode
 			{
 				Ogre::String newVal;
 				Ogre::TextureFilterOptions tfo;
 				unsigned int aniso;
 
-				switch (mDetailsPanel->getParamValue(9).asUTF8()[0])
+				switch (DISPLAY_STRING_TO_STRING(mDetailsPanel->getParamValue(9))[0])
 				{
 				case 'B':
 					newVal = "Trilinear";
@@ -218,37 +217,37 @@ namespace OgreBites
 			{
 				Ogre::TextureManager::getSingleton().reloadAll();
 			}
-			else if (evt.key == OIS::KC_SYSRQ)   // take a screenshot
+			else if (evt.key == OIS::KC_F6)   // take a screenshot
 			{
 				mWindow->writeContentsToTimestampedFile("screenshot", ".png");
 			}
 
-#ifdef USE_RTSHADER_SYSTEM		
+#ifdef INCLUDE_RTSHADER_SYSTEM		
 			// Toggle schemes.			
 			else if (evt.key == OIS::KC_F2)
 			{	
-				Ogre::Viewport* mainVP = mCamera->getViewport();
-				const Ogre::String& curMaterialScheme = mainVP->getMaterialScheme();
-
-				if (curMaterialScheme == Ogre::MaterialManager::DEFAULT_SCHEME_NAME)
-				{							
-					mainVP->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
-					mDetailsPanel->setParamValue(11, "On");
-				}
-				else if (curMaterialScheme == Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME)
+				if(mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION))
 				{
-					mainVP->setMaterialScheme(Ogre::MaterialManager::DEFAULT_SCHEME_NAME);
-					mDetailsPanel->setParamValue(11, "Off");
-				}														
+					Ogre::Viewport* mainVP = mCamera->getViewport();
+					const Ogre::String& curMaterialScheme = mainVP->getMaterialScheme();
+
+					if (curMaterialScheme == Ogre::MaterialManager::DEFAULT_SCHEME_NAME)
+					{							
+						mainVP->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
+						mDetailsPanel->setParamValue(11, "On");
+					}
+					else if (curMaterialScheme == Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME)
+					{
+						mainVP->setMaterialScheme(Ogre::MaterialManager::DEFAULT_SCHEME_NAME);
+						mDetailsPanel->setParamValue(11, "Off");
+					}														
+				}
 			}			
-
-#ifdef RTSHADER_SYSTEM_BUILD_EXT_SHADERS
-
 			// Toggles per pixel per light model.
 			else if (evt.key == OIS::KC_F3)
 			{
 				static bool usePerPixelLighting = true;					
-								
+												
 				// Grab the scheme render state.												
 				Ogre::RTShader::RenderState* schemRenderState = mShaderGenerator->getRenderState(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
 
@@ -293,7 +292,6 @@ namespace OgreBites
 					mDetailsPanel->setParamValue(12, "Vertex");
 				usePerPixelLighting = !usePerPixelLighting;				
 			}	
-#endif
 
 			// Switch vertex shader outputs compaction policy.
 			else if (evt.key == OIS::KC_F4)   
@@ -319,7 +317,7 @@ namespace OgreBites
 				// Invalidate the scheme in order to re-generate all shaders based technique related to this scheme.
 				mShaderGenerator->invalidateScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
 			}	
-#endif
+#endif // INCLUDE_RTSHADER_SYSTEM
 
 			mCameraMan->injectKeyDown(evt);
 			return true;
@@ -336,7 +334,7 @@ namespace OgreBites
 		to filter out any interface-related mouse events before processing them in your scene.
 		If the tray manager handler returns true, the event was meant for the trays, not you. */
 
-#if OGRE_PLATFORM == OGRE_PLATFORM_IPHONE
+#if (OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS) || (OGRE_PLATFORM == OGRE_PLATFORM_ANDROID)
 		virtual bool touchMoved(const OIS::MultiTouchEvent& evt)
 #else
 		virtual bool mouseMoved(const OIS::MouseEvent& evt)
@@ -349,7 +347,7 @@ namespace OgreBites
 			return true;
 		}
 
-#if OGRE_PLATFORM == OGRE_PLATFORM_IPHONE
+#if (OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS) || (OGRE_PLATFORM == OGRE_PLATFORM_ANDROID)
 		virtual bool touchPressed(const OIS::MultiTouchEvent& evt)
 		{
 			if (mTrayMgr->injectMouseDown(evt)) return true;
@@ -381,7 +379,7 @@ namespace OgreBites
 		}
 #endif
 
-#if OGRE_PLATFORM == OGRE_PLATFORM_IPHONE
+#if (OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS) || (OGRE_PLATFORM == OGRE_PLATFORM_ANDROID)
 		virtual bool touchReleased(const OIS::MultiTouchEvent& evt)
 		{
 			if (mTrayMgr->injectMouseUp(evt)) return true;
@@ -416,39 +414,21 @@ namespace OgreBites
 		/*-----------------------------------------------------------------------------
 		| Extended to setup a default tray interface and camera controller.
 		-----------------------------------------------------------------------------*/
-#if OGRE_PLATFORM == OGRE_PLATFORM_IPHONE
-		virtual void _setup(Ogre::RenderWindow* window, OIS::MultiTouch* mouse, FileSystemLayer* fsLayer)
-#else
-		virtual void _setup(Ogre::RenderWindow* window, OIS::Keyboard* keyboard, OIS::Mouse* mouse, FileSystemLayer* fsLayer)
-#endif
+		virtual void _setup(Ogre::RenderWindow* window, InputContext inputContext, Ogre::FileSystemLayer* fsLayer, Ogre::OverlaySystem* overlaySys)
 		{
 			// assign mRoot here in case Root was initialised after the Sample's constructor ran.
 			mRoot = Ogre::Root::getSingletonPtr();
 			mWindow = window;
-#if OGRE_PLATFORM != OGRE_PLATFORM_IPHONE
-			mKeyboard = keyboard;
-#endif
-			mMouse = mouse;
+			mInputContext = inputContext;
 			mFSLayer = fsLayer;
+			mOverlaySystem = overlaySys;
 
 			locateResources();
 			createSceneManager();
 			setupView();
 
-			mTrayMgr = new SdkTrayManager("SampleControls", window, mouse, this);  // create a tray interface
-			
-#ifdef USE_RTSHADER_SYSTEM
-			// Initialize shader generator.
-			// Must be before resource loading in order to allow parsing extended material attributes.
-			bool success = initializeRTShaderSystem(mSceneMgr);
-			if (!success) 
-			{
-				OGRE_EXCEPT(Ogre::Exception::ERR_FILE_NOT_FOUND, 
-					"Shader Generator Initialization failed - Core shader libs path not found", 
-					"SdkSample::_setup");
-			}														
-#endif
-			
+			mTrayMgr = new SdkTrayManager("SampleControls", window, inputContext, this);  // create a tray interface
+
 			loadResources();
 			mResourcesLoaded = true;
 
@@ -471,12 +451,17 @@ namespace OgreBites
 			items.push_back("Filtering");
 			items.push_back("Poly Mode");
 
-#ifdef USE_RTSHADER_SYSTEM
+#ifdef INCLUDE_RTSHADER_SYSTEM
 			items.push_back("RT Shaders");
 			items.push_back("Lighting Model");
 			items.push_back("Compact Policy");
 			items.push_back("Generated VS");
 			items.push_back("Generated FS");														
+
+			// fix scene compositor for d3d11
+			// Ogre::CompositorManager& compMgr = Ogre::CompositorManager::getSingleton();
+			// Ogre::CompositorPtr scene = compMgr.getByName("Ogre/Scene/Default", Ogre::ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
+			// scene->getTechnique(0)->getOutputTargetPass()->setMaterialScheme(Ogre::Root::getSingleton().getRenderSystem()->_getDefaultViewportMaterialScheme());
 #endif
 
 			mDetailsPanel = mTrayMgr->createParamsPanel(TL_NONE, "DetailsPanel", 200, items);
@@ -485,8 +470,16 @@ namespace OgreBites
 			mDetailsPanel->setParamValue(9, "Bilinear");
 			mDetailsPanel->setParamValue(10, "Solid");
 
-#ifdef USE_RTSHADER_SYSTEM
+#ifdef INCLUDE_RTSHADER_SYSTEM
 			mDetailsPanel->setParamValue(11, "Off");
+
+            Ogre::Viewport* mainVP = mCamera->getViewport();
+            //const Ogre::String& curMaterialScheme = mainVP->getMaterialScheme();
+            if(mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION) == false)
+            {
+                mainVP->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
+                mDetailsPanel->setParamValue(11, "On");
+            }
 			mDetailsPanel->setParamValue(12, "Vertex");
 			mDetailsPanel->setParamValue(13, "Low");
 			mDetailsPanel->setParamValue(14, "0");
@@ -503,7 +496,7 @@ namespace OgreBites
 		{
 			Sample::_shutdown();
 
-			if (mTrayMgr) delete mTrayMgr;
+            if (mTrayMgr) delete mTrayMgr;
 			if (mCameraMan) delete mCameraMan;
 
 			// restore settings we may have changed, so as not to affect other samples
@@ -519,6 +512,7 @@ namespace OgreBites
 			mCamera = mSceneMgr->createCamera("MainCamera");
 			mViewport = mWindow->addViewport(mCamera);
 			mCamera->setAspectRatio((Ogre::Real)mViewport->getActualWidth() / (Ogre::Real)mViewport->getActualHeight());
+            mCamera->setAutoAspectRatio(true);
 			mCamera->setNearClipDistance(5);
 
 			mCameraMan = new SdkCameraMan(mCamera);   // create a default camera controller
